function [pl_model, sl_model, mu_pl, mu_sl] = EUPAL_train(labeled_data, labeled_target, pl_data, pl_target, alpha, T)

% Syntax
%
%       [pl_model, sl_model, mu_pl, mu_sl] = EUPAL_train(labeled_data, labeled_target, pl_data, pl_target, alpha, T)
%
% Description
%
%       EUPAL takes,
%           labeled_data        - A LxD array, the ith instance of labeled instance is stored in labeled_data(i,:)
%           labeled_target      - A LxQ array, if the jth class label is the ground-truth labels for the ith labeled instance, then labeled_target(i,j) equals +1, otherwise labeled_target(i,j) equals 0
%           pl_data             - A UxD array, the ith instance of unlabeled instance is stored in pl_data(i,:)
%           pl_target           - A UxQ array, if the jth class label is the candidate labels for the ith unlabeled instance, then pl_target(i,j) equals +1, otherwise pl_target(i,j) equals 0
%           alpha               - The balancing parameter with 0<alpha<1 (defalut 0.4)
%           T                   - The maximum number of iterations (defalut 50)
%      
%       and returns,
%           pl_model            - The partial label training model
%           sl_model            - The supervised training model
%           mu_pl               - The weight of partial label training model
%           mu_sl               - The weight of supervised training model

if nargin<6
    T=50;
end
if nargin<5
    alpha=0.4;
end
if nargin<4
    error('Not enough input parameters, please check again.');
end
if size(labeled_data,1)~=size(labeled_target,1) || size(pl_data,1)~=size(pl_target,1)
    error('Length of label vector does match the number of instances');
end

options='-t 2 -c 1 -b 1';

L = size(labeled_data,1);
U = size(pl_data,1);

n = max(round(U/T),3);
class_num = size(labeled_target,2);

sl_data = labeled_data;
[~,sl_label]=max(labeled_target,[],2);

su = zeros(U,class_num);
diff = inf;
iter = 0;
while iter < T && abs(diff) > 0.0001
    last_su = su;

    pl_model = IPAL_train(pl_data, pl_target');
    sl_model = libsvmtrain(sl_label, sl_data, options);
    
    su = pl_model.finalConfidence';
    choice2 = randperm(U, n);
    delta2_data = pl_data(choice2, :);
    [~, ~, dec_values] = libsvmpredict(zeros(n, 1), delta2_data, sl_model, '-b 1');
    su(choice2, :) = alpha*dec_values + (1-alpha)*su(choice2, :);
    pl_target(su>=(1/class_num))=1;
    pl_target(su<(1/class_num))=0;

    choice1 = randperm(U, n);
    delta1_data = pl_data(choice1, :);
    delta1_label = pl_model.disambiguatedLabel(:, choice1);
    for i = 1:n
        [judge, row_index] = ismember(delta1_data(i, :), sl_data, 'rows'); 
        if judge
            sl_label(row_index, :) = find(delta1_label(:, i) == 1);
        else
            sl_data = [sl_data; delta1_data(i, :)];
            sl_label = [sl_label; find(delta1_label(:, i) == 1)];
        end
    end

    diff = norm(last_su - su, 2);
    iter = iter+1;
end

pl_model = IPAL_train(pl_data, pl_target');
[mu_pl] = IPAL_predict(labeled_data, labeled_target', pl_model);

sl_model = libsvmtrain(sl_label, sl_data, options);
[~,sl_label]=max(labeled_target,[],2);
[~, sl_acc,~] = libsvmpredict(sl_label, labeled_data, sl_model, '-b 1');
mu_sl = sl_acc(1)/100;
end